﻿using System;
using System.IO;
using System.Text.RegularExpressions;
using System.Web;

namespace Microsoft.Research.RMCr
{
    /// <summary>
    /// An HttpModule which redirects an image request to a JPEGXR image if one exists and 
    /// the user's browser supports JPEGXR images.
    /// </summary>
    /// <remarks>Currently, only IE10 supports JPEGXR images.</remarks>
    public class JpegXrModule : IHttpModule
    {
        private static Regex _browserVersionRegex = new Regex(@"MSIE (?<majorVersion>[0-9]{1,})[\.0-9]{0,}");

        private const string JpegMimeType = "image/jpeg";
        private const string JpegXrExtension = ".jxr";
        private const string PngMimeType = "image/png";

#if DEBUG
        private const int MinimumIeVersion = 9;
#else
        private const int MinimumIeVersion = 10;
#endif

        #region IHttpModule members

        public void Dispose()
        {
            // Deliberately empty
        }

        public void Init(HttpApplication context)
        {
            // Wire up the handler
            context.BeginRequest += new EventHandler(context_BeginRequest);
        }

        #endregion

        /// <summary>
        /// Handles the BeginRequest event.
        /// </summary>
        void context_BeginRequest(object sender, EventArgs e)
        {
            HttpRequest request = HttpContext.Current.Request;
            HttpResponse response = HttpContext.Current.Response;

            // Only process image requests where the user's browser supports JPEGXR
            string mimeType = GetMimeType(request.PhysicalPath);

            if ((mimeType == JpegMimeType || mimeType == PngMimeType) && BrowserSupportsJpegXr(request.UserAgent))
            {
                // Get the full physical path to the image
                string imageFile = request.PhysicalPath;

                // Is there a JPEGXR file in the same location?
                string jpegXrFile = Path.Combine(Path.GetDirectoryName(imageFile), Path.GetFileNameWithoutExtension(imageFile) + JpegXrExtension);

                if (File.Exists(jpegXrFile))
                {
                    HttpContext.Current.RewritePath(Path.GetFileNameWithoutExtension(request.Path) + JpegXrExtension, String.Empty, String.Empty, true);
                }
            }
        }

        /// <summary>
        /// Checks to see if the user's browser supports JPEGXR images.
        /// </summary>
        /// <param name="userAgent">The user-agent string of the browser.</param>
        /// <returns>True if the browser supports JPEGXR images, otherwise false.</returns>
        private static bool BrowserSupportsJpegXr(string userAgent)
        {
            bool supportsJpegXr = false;

            try
            {
                // First, check for Mozilla browser
                if (userAgent.StartsWith("Mozilla/", StringComparison.OrdinalIgnoreCase))
                {
                    // Next, check IE version
                    int browserMajor;
                    Match browserVersionMatch = _browserVersionRegex.Match(userAgent);

                    if (browserVersionMatch != null &&
                        int.TryParse(browserVersionMatch.Groups["majorVersion"].Value, out browserMajor) &&
                        browserMajor >= MinimumIeVersion)
                    {
                        supportsJpegXr = true;
                    }
                }
            }
            catch { }

            return (supportsJpegXr);
        }

        /// <summary>
        /// Gets the MIME type for the currently requested file from the registry.
        /// </summary>
        /// <param name="fileName">The file for which the MIME type is required.</param>
        /// <returns>The MIME type for the file.</returns>
        private static string GetMimeType(string fileName)
        {
            // Default MIME type
            string mimeType = "application/octetstream";

            try
            {
                Microsoft.Win32.RegistryKey regKey = Microsoft.Win32.Registry.ClassesRoot.OpenSubKey(Path.GetExtension(fileName).ToLowerInvariant());
                if (regKey != null)
                {
                    object contentType = regKey.GetValue("Content Type");

                    if (contentType != null)
                        mimeType = contentType.ToString();
                }
            }
            catch { }

            return (mimeType);
        }
    }
}
